@@ -7,7 +7,7 @@
 #' marginal AIC), i.e., including degrees of freedom from the
 #' smooth/random effect components of the model.
 #' 
-#' @param object SDE model object
+#' @param object HMM model object
 #' @param ... For compatibility with S3 method
 #' 
 #' @return Maximum log-likelihood value for the model, with attributes
@@ -15,6 +15,11 @@
 #' 
 #' @export
 logLik.HMM <- function(object, ...) {
+  if(nrow(object$obs()$coeff_re()) + nrow(object$hid()$coeff_re()) > 0) {
+    warning("AIC/BIC functions are experimental for models with random effects",
+            " or splines. Use at your own risk.")
+  }
+  
   par_all <- c(object$tmb_rep()$par.fixed, 
                object$tmb_rep()$par.random)
   val <- - object$tmb_obj_joint()$fn(par_all)
