@@ -52,7 +52,9 @@ MarkovChain <- R6Class(
     #' example, ref = c(1, 1) means that the first element of the first row
     #' Pr(1>1) and the first element of the second row Pr(2>1) are used as 
     #' reference elements and are not estimated. If this is not provided,
-    #' the diagonal transition probabilities are used as references. 
+    #' the diagonal transition probabilities are used as references.
+    #' @param gam_args Named list of arguments passed to \code{mgcv::gam()} in
+    #' \code{MarkovChain$make_mat()}, e.g., "knots". Use at your own risk.
     #' 
     #' @return A new MarkovChain object
     #' 
@@ -76,13 +78,14 @@ MarkovChain <- R6Class(
     #'                     ncol = 2, byrow = TRUE)
     #' hid <- MarkovChain$new(data = energy, n_states = 2,
     #'                        formula = structure)
-    initialize = function(data,
+    initialize = function(data = NULL,
                           formula = NULL, 
                           n_states,
                           tpm = NULL,
                           initial_state = "estimated",
                           fixpar = NULL,
-                          ref = 1:n_states) {
+                          ref = 1:n_states,
+                          gam_args = NULL) {
       # Check arguments
       private$check_args(n_states = n_states, 
                          formula = formula, 
@@ -90,6 +93,20 @@ MarkovChain <- R6Class(
       private$nstates_ <- n_states
       private$ref_ <- ref
       
+      # If no data is passed, create data frame with two rows (minimum
+      # required by mgcv::gam for initialisation)
+      if(is.null(data)) {
+        message(paste("No 'data' argument -- creating empty model", 
+                      "(should be used for simulation only)"))
+        data <- data.frame(ID = c(1, 1))
+        private$empty_ <- TRUE
+      } else {
+        private$empty_ <- FALSE
+      }
+      
+      # Save user-specified arguments for mgcv::gam()
+      private$gam_args_ <- gam_args
+      
       # Matrix with 1 for reference element and 0 elsewhere
       # (used later to select relevant entries of tpm)
       private$ref_mat_ <- matrix(0, n_states, n_states)
@@ -177,7 +194,7 @@ MarkovChain <- R6Class(
       # Create terms and necessary matrices 
       mats <- self$make_mat(data = data)
       ncol_fe <- mats$ncol_fe
-      ncol_re <- mats$ncol_re       
+      ncol_re <- mats$ncol_re
       private$terms_ <- c(mats, list(names_fe = colnames(mats$X_fe),
                                      names_re_all = colnames(mats$X_re),
                                      names_re = colnames(ncol_re)))
@@ -392,6 +409,12 @@ MarkovChain <- R6Class(
     #' @description Initial state (see constructor argument)
     initial_state = function() {return(private$initial_state_)},
     
+    #' @description Empty model? (for simulation only)
+    empty = function() {return(private$empty_)},
+    
+    #' @description Extra arguments for mgcv::gam (passed to make_matrices)
+    gam_args = function() {return(private$gam_args_)},
+    
     # Mutators ----------------------------------------------------------------
     
     #' @description Update transition probability matrix
@@ -534,7 +557,8 @@ MarkovChain <- R6Class(
     make_mat = function(data, new_data = NULL) {
       make_matrices(formulas = self$formulas(), 
                     data = data, 
-                    new_data = new_data)
+                    new_data = new_data,
+                    gam_args = self$gam_args())
     },
     
     #' @description Design matrices for grid of covariates
@@ -707,6 +731,8 @@ MarkovChain <- R6Class(
     fixpar_user_ = NULL,
     fixpar_ = NULL,
     initial_state_ = NULL,
+    empty_ = NULL,
+    gam_args_ = NULL,
     
     # Setup fixed parameters
     setup_fixpar = function() {
